###################################################################################
# Phabricator QA tool / Task command.
# Copyright (C) 2015, 2016  Luis Araujo <luis.araujo@collabora.co.uk>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  US
###################################################################################

from pqa_tool.commands import Command, _get_date

class TaskCmd(Command):

    def __init__(self, args):
        Command.__init__(self, args)

    def run(self):
        result = self.call('maniphest.info', task_id=self.args.task_id)
        title = result['objectName'] + " - " + result['title']
        print title + " [{}]".format(result['status'].upper())
        print "=" * len(title)
        print result['uri']
        print
        owner_phid = result.get('ownerPHID')
        if owner_phid:
            print "Assigned To:", \
                self._phid_query([owner_phid])[owner_phid]['fullName']

        print "Priority:", result['priority']

        author_phid = result.get('authorPHID')
        if author_phid:
            print "Author:", \
                self._phid_query([author_phid])[author_phid]['fullName']

        depends_phids = result.get('dependsOnTaskPHIDs')
        if depends_phids:
            for task in self._phid_query(depends_phids).itervalues():
                print "Blocked By:", task['fullName'], \
                    "[{}] ({})".format(task['status'], task['uri'])

        cc_phids = result.get('ccPHIDs')
        if cc_phids:
            print "Subscribers:", \
                ", ".join([ member['fullName'] for member in \
                            self._phid_query(cc_phids).itervalues() ])

        project_phids = result.get('projectPHIDs')
        if project_phids:
            print "Projects:", \
                ", ".join([ project['fullName'] for project in \
                            self._phid_query(project_phids).itervalues() ])

        print "Date Created:", _get_date(result['dateCreated'])
        print "Last Modified:", _get_date(result['dateModified'])
        print
        print result['description']

        # Show comments if --show-comments passed.
        if self.args.show_comments:
            realnames = {}
            i = 0
            # Query comments from task transactions.
            trq = self.call('maniphest.gettasktransactions', \
                            ids=[self.args.task_id])
            # Reverse so it gets ordered from older to newer (by date created).
            transactions = trq[str(self.args.task_id)]
            transactions.reverse()
            for transaction in transactions:
                if transaction['transactionType'] == "core:comment":
                    i += 1
                    authorphid = transaction['authorPHID']
                    # Cache real names to avoid keep calling every time.
                    if not realnames.get(authorphid, None):
                        userq = self.call('user.query', phids=[authorphid])
                        realnames.update({ authorphid: userq.pop()['realName'] })

                    date = _get_date(transaction['dateCreated'])
                    # Print comment
                    print ("\n----- Comment {} / " + realnames[authorphid] + \
                               " ({}) -----").format(i, date)
                    print transaction['comments']
