###################################################################################
# Phabricator QA tool. Commands module.
# Copyright (C) 2015, 2016  Luis Araujo <luis.araujo@collabora.co.uk>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  US
###################################################################################

import json
import os.path

from pyconduit import Conduit, ConfigException

class Command(object):
    """Base object for all commands"""

    def __init__(self, args):
        self.args = args
        try:
            self.conduit = Conduit.from_config(args.config)
        except ConfigException as e:
            print "{}: {}".format(e, e.value)
            exit(1)

    def call(self, method, **params):
        """This method calls the conduit api method with params."""
        content = self.conduit.call(method, **params)
        if content['error_code']:
            print content['error_info']
            exit(2)
        return content['result']

    def _phid_for_user(self, username):
        """Return the PHID for username"""
        userq = self.call('user.query', usernames=[username])
        if not userq:
            print "No user {}".format(username)
            exit(2)
        return userq.pop()['phid']

    def _phid_for_project(self, projectname):
        """Return the PHID for projectname"""
        projectq = self.call('project.query', names=[projectname])
        if not projectq.get('data', None):
            print "No project {}".format(projectname)
            exit(2)
        return projectq['data'].popitem()[0]

    def _phid_query(self, phids):
        """Take a list of PHIDs and return hash of found objects"""
        return self.call('phid.query', phids=phids)

#
# Utility functions used by commands.
#
def _priority_int(pstr):
    return { 'low':     50,
             'normal':  60,
             'triage':  70,
             'high':    90,
             'blocker': 100 }[pstr]

def _get_date(date):
    import datetime
    return datetime.datetime.utcfromtimestamp(float(date))

def _build_auxiliary_fields(fields):
    kv_fields = {}
    for f in fields:
        try:
            k, v = f.split('=', 1)
        except ValueError:
            print "Not valid field:", f
            exit(1)
        kv_fields.update({ k: v })
    return kv_fields
