###################################################################################
# Phabricator QA tool / Create task command.
# Copyright (C) 2015, 2016  Luis Araujo <luis.araujo@collabora.co.uk>

# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.

# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  US
###################################################################################

from pqa_tool.commands import \
    Command, _build_auxiliary_fields, _priority_int

class CreateTaskCmd(Command):

    def __init__(self, args):
        Command.__init__(self, args)

    def run(self):
        params = { 'title': self.args.title }

        if self.args.description:
            params.update({ 'description': self.args.description })

        if self.args.owner:
            params.update({ 'ownerPHID': self._phid_for_user(self.args.owner) })

        if self.args.member:
            params.update({ 'ccPHIDs': \
                            [ self._phid_for_user(m) for m in self.args.member ] })

        if self.args.priority:
            params.update({ 'priority': _priority_int(self.args.priority) })

        if self.args.project:
            params.update({ 'projectPHIDs': [ self._phid_for_project(p)
                                              for p in self.args.project ] })

        if self.args.field:
            params.update({ 'auxiliary':
                            _build_auxiliary_fields(self.args.field) })

        result = self.call('maniphest.createtask', **params)
        print "Task created: {}".format(result['uri'])
